<?php
/***************************************************************
*  Copyright notice
*  
*  (c) 2004 Mathias Bolt Lesniak <mathias@lilio.com>
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is 
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
* 
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*  A copy is found in the textfile GPL.txt and important notices to the license 
*  from the author is found in LICENSE.txt distributed with these scripts.
*
* 
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/**
 * Adds event capabilities to tt_news.
 * Listing partly built on elements from tt_news by Kasper Skrhj <kasper@typo3.com>
 *
 * @author	Mathias Bolt Lesniak <mathias@lilio.com>
 */
require_once(PATH_tslib."class.tslib_pibase.php");


class tx_mblnewsevent extends tslib_pibase {
	var $prefixId = "tx_mblnewsadd";		// Same as class name
	var $scriptRelPath = "class.mblnewsevent.php";	// Path to this script relative to the extension dir.
	var $extKey = "mbl_newsevent";	// The extension key.
	
	//Default configuration variables
	var $eventTemplateFile = "EXT:mbl_newsevent/event_template.tmpl";
	var $whereStdWrap = array('wrap'=>" | ");
	var $dateStdWrap = array('strftime'=>"%x");
	var $timeStdWrap = array('strftime'=>"%H:%M");
	var $dateSelMode = 'MONTH';
	var $showArticleCountInDateSel = 1;
	var $dateSelItemStdWrap = array('strftime'=>"%B - %Y");
	var $eventPeriodTarget = "_self";
	var $latestShowsNewEvents = 0;
	var $icsTemplateFile = "EXT:mbl_newsevent/res/ics_template.tmpl";
	var $icsFromDate = "-1 year";
	var $icsToDate = "+1 year";
	var $icsCalendarName = "Event Calendar";
	var $eventDateCorrect = 0;
	var $organizerStdWrap = array('wrap'=>" | ");
	var $showOrganizerEmail = true;
	var $icsFileName = "Event Calendar.ics";
		
	//For ICS download
	function icsOutput($parentObject) {
		$where=" AND tt_news.tx_mblnewsevent_isevent = 1";
		$where .= " AND tt_news.tx_mblnewsevent_from >= " . strtotime((empty($parentObject->conf["mbl_newsevent."]["ics."]["from"]))? $this->icsFromDate : $parentObject->conf["mbl_newsevent."]["ics."]["from"]) . " AND tt_news.tx_mblnewsevent_from <= " . strtotime((empty($parentObject->conf["mbl_newsevent."]["ics."]["to"]))? $this->icsToDate : $parentObject->conf["mbl_newsevent."]["ics."]["to"]) . " ";
		/*echo $where;
		echo "#" . date('Ymd', 1126821600);
		echo "#" . date('Ymd', strtotime($this->icsFromDate));
		echo "#" . date('Ymd', strtotime($this->icsToDate));*/
		$selectConf = $parentObject->getSelectConf($where, true);
		$selectConf['selectFields'] = '*';
		
		header('Content-type:text/calendar');
		//header('Content-type:text/plain');
		header('Content-Disposition: attachment; filename="' . (empty($parentObject->conf["mbl_newsevent."]["ics."]["fileName"])? $this->icsFileName : $parentObject->conf["mbl_newsevent."]["ics."]["fileName"]) . '"');
		
		$res = $parentObject->cObj->exec_getQuery('tt_news', $selectConf);
		
		//Get template
		$templateData = $parentObject->cObj->fileResource((empty($parentObject->conf["mbl_newsevent."]["ics."]["templateFile"])? $this->icsTemplateFile : $parentObject->conf["mbl_newsevent."]["ics."]["templateFile"]));
		
		//Add headers
		$header = $parentObject->cObj->getSubpart($templateData, "###HEADER###");
		
		//Make header marker array
		$headerMarkerArray = array();
		$headerMarkerArray['###CALENDAR_NAME###'] = $this->conf["mbl_newsevent."]["ics."]["calendarName"];
		
		//Substitute
		$header = $parentObject->cObj->substituteMarkerArray($header, $headerMarkerArray);
		$templateData = $parentObject->cObj->substituteSubpart($templateData, '###HEADER###', $header);
		
		//Add elements
		$seq = 0; //Sequence number
		$element = $parentObject->cObj->getSubpart($templateData, "###ELEMENT###");
		while($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res)) {
			//For some strange reason dates come out minus one day. Here's a hack to fix it:
			$row["tx_mblnewsevent_from"] += (empty($parentObject->conf['mbl_newsevent.']['dateCorrect'])? $this->eventDateCorrect : $parentObject->conf['mbl_newsevent.']['dateCorrect']);
			$row["tx_mblnewsevent_to"] += (empty($parentObject->conf['mbl_newsevent.']['dateCorrect'])? $this->eventDateCorrect : $parentObject->conf['mbl_newsevent.']['dateCorrect']);

			//Make marker array
			$markerArray = array();
			//If time of day is zero, count on alldayevent
			if(date('His', $row['tx_mblnewsevent_from']) == "000000" && date('His', $row['tx_mblnewsevent_to']) == "000000") {
				$markerArray['###START_DATE###'] = "DTSTART;VALUE=DATE:" . date('Ymd', $row['tx_mblnewsevent_from']);
				$markerArray['###END_DATE###'] = "DTSTART;VALUE=DATE:" . date('Ymd', $row['tx_mblnewsevent_to']);
			} else {
				$markerArray['###START_DATE###'] = "DTSTART:" . date('Ymd\THis\Z', $row['tx_mblnewsevent_from']);
				$markerArray['###END_DATE###'] = "DTEND:" . date('Ymd\THis\Z', $row['tx_mblnewsevent_to']);
			}
			$markerArray['###CREATED_DATE###'] = date('Ymd\THis\Z', $row['crdate']);
			$markerArray['###TITLE###'] = $this->icsEscape($row['title']);
			$markerArray['###BLURB###'] = $this->icsEscape($row['short']);
			$markerArray['###LOCATION###'] = $this->icsEscape($row['tx_mblnewsevent_where']);
			$markerArray['###UID###'] = md5($row['uid'] . "@" . $_SERVER['HTTP_HOST']);
			$markerArray['###SEQUENCE###'] = $seq;
			if ($row['type']) { // News type article or external url
				$parentObject->local_cObj->setCurrentVal($row['type'] == 1 ? $row['page'] : $row['ext_url']);
				$markerArray['###EVENT_URL###'] = $this->icsEscape($parentObject->local_cObj->typolinkWrap($parentObject->conf['pageTypoLink.']));
			} else {
				$markerArray['###EVENT_URL###'] = $this->icsEscape(t3lib_div::getIndpEnv('TYPO3_SITE_URL') . $parentObject->pi_getPageLink($parentObject->config['singlePid'], '', array('tt_news'=>$row['uid'],'backPid'=>$parentObject->config['backPid']),$parentObject->allowCaching,'',$parentObject->config['singlePid']));

			}
			//Organizer info
			if(!empty($row['tx_mblnewsevent_organizer'])) {
				$table = substr($row['tx_mblnewsevent_organizer'], 0, strrpos($row['tx_mblnewsevent_organizer'], '_')); //The table to select from
				$sql = "SELECT * FROM " . $table . " WHERE uid = " . substr($row['tx_mblnewsevent_organizer'], strrpos($row['tx_mblnewsevent_organizer'], '_')+1);
				
				$ores = mysql_query($sql);
				//If there is any organizer
				if($res && mysql_num_rows($ores) != 0) {
					//Get row
					$orow = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($ores);
					//If organizer should be wrapped in e-mail link
					if(!empty($orow['email']) && ((empty($parentObject->conf["mbl_newsevent."]["showOrganizerEmail"]) && $this->showOrganizerEmail) || (!empty($parentObject->conf["mbl_newsevent."]["showOrganizerEmail"]) && $parentObject->conf["mbl_newsevent."]["showOrganizerEmail"]))) {
						$markerArray['###ORGANIZER_NAME###'] = $this->icsEscape(empty($orow['name'])? $orow['realName'] : $orow['name']);
						$markerArray['###ORGANIZER_EMAIL###'] = $this->icsEscape($orow['email']);
					} else {
						$markerArray['###ORGANIZER_NAME###'] = $this->icsEscape(empty($orow['name'])? $orow['realName'] : $orow['name']);
						$markerArray['###ORGANIZER_EMAIL###'] = "";
					}
				} else {
					$markerArray['###ORGANIZER_NAME###'] = "";
					$markerArray['###ORGANIZER_EMAIL###'] = "";
				}
			} else {
				$markerArray['###ORGANIZER_NAME###'] = "";
				$markerArray['###ORGANIZER_EMAIL###'] = "";
			}
			
			$elements .=  $parentObject->cObj->substituteMarkerArray($element, $markerArray);
			
			$seq++;
		}
		$templateData = $parentObject->cObj->substituteSubpart($templateData, '###ELEMENT###', $elements);
		
		
		return $templateData;
	}
	
	function icsEscape($s) {
		$original = array('\\', ';', ',', "\n", "\r");
		$replace = array('\\\\', '\;', '\,', "\\n", "\\r");
		
		return str_replace($original, $replace, $s);
	}
	
	function extraItemMarkerProcessor($markerArray,$row,$lConf,$parentObject) {
		//Initiate language
		$this->pi_loadLL();
		
		
		//For some strange reason dates come out minus one day. Here's a hack to fix it:
		$row["tx_mblnewsevent_from"] += $parentObject->conf['mbl_newsevent.']['dateCorrect'];
		$row["tx_mblnewsevent_to"] += $parentObject->conf['mbl_newsevent.']['dateCorrect'];
				
		if($row['tx_mblnewsevent_isevent'] != 0) {
			//Get event template
			$eventTemplateCode = $parentObject->cObj->fileResource($parentObject->conf["mbl_newsevent."]["templateFile"]);
			
			//For EVENT_WRAP
			//If it's the same day
			if($parentObject->local_cObj->stdWrap($row["tx_mblnewsevent_from"],$lConf["date_stdWrap."]) == $parentObject->local_cObj->stdWrap($row["tx_mblnewsevent_to"],$lConf["date_stdWrap."])) {
				//Get template
				$eventWrap = $parentObject->cObj->getSubpart($eventTemplateCode,/*$parentObject->spMarker(*/"###ONE_DAY_EVENT###"/*)*/);
			
			//If it's not the same day
			} else {
				//Get template
				$eventWrap = $parentObject->cObj->getSubpart($eventTemplateCode,/*$parentObject->spMarker(*/"###MULTIPLE_DAY_EVENT###"/*)*/);			
			}
			
			//set values
			$eventmarkerArray['###EVENT_FROM_DATE###'] = $parentObject->local_cObj->stdWrap($row["tx_mblnewsevent_from"],(empty($lConf["mbl_newsevent."]["date_stdWrap."])? $parentObject->conf["mbl_newsevent."]["date_stdWrap."] : $lConf["mbl_newsevent."]["date_stdWrap."]));
			$eventmarkerArray['###EVENT_FROM_TIME###'] = $parentObject->local_cObj->stdWrap($row["tx_mblnewsevent_from"],(empty($lConf["mbl_newsevent."]["date_stdWrap."])? $parentObject->conf["mbl_newsevent."]["time_stdWrap."] : $lConf["mbl_newsevent."]["time_stdWrap."]));
			$eventmarkerArray['###EVENT_TO_DATE###'] = $parentObject->local_cObj->stdWrap($row["tx_mblnewsevent_to"],(empty($lConf["mbl_newsevent."]["date_stdWrap."])? $parentObject->conf["mbl_newsevent."]["date_stdWrap."] : $lConf["mbl_newsevent."]["date_stdWrap."]));
			$eventmarkerArray['###EVENT_TO_TIME###'] = $parentObject->local_cObj->stdWrap($row["tx_mblnewsevent_to"],(empty($lConf["mbl_newsevent."]["date_stdWrap."])? $parentObject->conf["mbl_newsevent."]["time_stdWrap."] : $lConf["mbl_newsevent."]["time_stdWrap."]));
			$eventmarkerArray['###EVENT_WHERE###'] = $parentObject->local_cObj->stdWrap($row["tx_mblnewsevent_where"],$lConf["mbl_newsevent."]["where_stdWrap."]);
			$eventmarkerArray['###EVENT_DATE_TEXT###'] = $this->pi_getLL("event_date");
			$eventmarkerArray['###EVENT_DATES_TEXT###'] = $this->pi_getLL("event_dates");
			$eventmarkerArray['###EVENT_WHERE_TEXT###'] = $this->pi_getLL("location");
			//Organizer info
			if(!empty($row['tx_mblnewsevent_organizer'])) {
				$table = substr($row['tx_mblnewsevent_organizer'], 0, strrpos($row['tx_mblnewsevent_organizer'], '_')); //The table to select from
				$sql = "SELECT * FROM " . $table . " WHERE uid = " . substr($row['tx_mblnewsevent_organizer'], strrpos($row['tx_mblnewsevent_organizer'], '_')+1);
				
				$res = mysql_query($sql);
				//If there is any organizer
				if($res && mysql_num_rows($res) != 0) {
					//Get row
					$orow = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res);
					//If organizer should be wrapped in e-mail link
					if(!empty($orow['email']) && ($parentObject->conf["mbl_newsevent."]["showOrganizerEmail"])) {
						$eventmarkerArray['###EVENT_ORGANIZER###'] = '<a href="mailto:' . $orow['email'] . '">' . $parentObject->local_cObj->stdWrap((empty($orow['name'])? $orow['realName'] : $orow['name']),$lConf["mbl_newsevent."]["organizer_stdWrap."]) . '</a>';
					} else {
						$eventmarkerArray['###EVENT_ORGANIZER###'] = $parentObject->local_cObj->stdWrap((empty($orow['name'])? $orow['realName'] : $orow['name']),$lConf["mbl_newsevent."]["organizer_stdWrap."]);
					}
				} else {
					$eventmarkerArray['###EVENT_ORGANIZER###'] = "";
				}
			} else {
				$eventmarkerArray['###EVENT_ORGANIZER###'] = "";
			}
			$eventmarkerArray['###EVENT_ORGANIZER_TEXT###'] = $this->pi_getLL("organizer");
			$eventmarkerArray['###EVENT_TO_TEXT###'] = $this->pi_getLL("to");
			
			//FOR WHERE_WRAP
			$whereWrap = $parentObject->cObj->getSubpart($eventTemplateCode,$parentObject->spMarker("###EVENT_WHERE_WRAP###"));
			$wheremarkerArray = $eventmarkerArray;
					
			//Insert
			$markerArray['###EVENT_WRAP###'] = $parentObject->cObj->substituteMarkerArrayCached($eventWrap,$eventmarkerArray,array(),array());
			$markerArray['###EVENT_WHERE_WRAP###'] = $parentObject->cObj->substituteMarkerArrayCached($whereWrap,$wheremarkerArray,array(),array());
			$markerArray['###EVENT_FROM_DATE###'] = $eventmarkerArray['###EVENT_FROM_DATE###'];
			$markerArray['###EVENT_FROM_TIME###'] = $eventmarkerArray['###EVENT_FROM_TIME###'];
			$markerArray['###EVENT_TO_DATE###'] = $eventmarkerArray['###EVENT_TO_DATE###'];
			$markerArray['###EVENT_TO_TIME###'] = $eventmarkerArray['###EVENT_TO_TIME###'];
			$markerArray['###EVENT_WHERE###'] = $parentObject->local_cObj->stdWrap($row["tx_mblnewsevent_where"],$lConf["mbl_newsevent."]["where_stdWrap."]);
			$markerArray['###EVENT_DATE_TEXT###'] = $this->pi_getLL("event_date");
			$markerArray['###EVENT_DATES_TEXT###'] = $this->pi_getLL("event_dates");
			$markerArray['###EVENT_WHERE_TEXT###'] = $this->pi_getLL("location");
			$markerArray['###EVENT_ORGANIZER###'] = $eventmarkerArray['###EVENT_ORGANIZER###'];
			$markerArray['###EVENT_ORGANIZER_TEXT###'] = $eventmarkerArray['###EVENT_ORGANIZER_TEXT###'];
			$markerArray['###EVENT_TO_TEXT###'] = $eventmarkerArray['###EVENT_TO_TEXT###'];
		} else {
			$markerArray['###EVENT_WRAP###'] = "";
			$markerArray['###EVENT_WHERE_WRAP###'] = "";
		}
						
		//For beta compatibility
		$markerArray['###EVENT###'] = "";
				
		return $markerArray;
	}
	
	function makeDateSel($parentObject) {
		//Get template
		$eventTemplateCode = $parentObject->cObj->fileResource(((empty($parentObject->conf["mbl_newsevent."]["templateFile"]))? $parentObject->conf["eventTemplateFile"] : $parentObject->conf["mbl_newsevent."]["templateFile"]));
		
		//Get template section
		$template = $parentObject->cObj->getSubpart($eventTemplateCode,"###DATESEL_LIST###");
		
		$itemTemplate = $parentObject->cObj->getSubpart($template,"###ITEM###");
		
		//Get the date selection mode
		$mode = strtolower($parentObject->conf['mbl_newsevent.']['dateSelMode']);
		//Check if the mode is correct
		if(!in_array('$mode', array('month', 'quarter', 'year'))) {
			return array('###EVENT_SELECT_LIST###'=>"<p>Wrong mode: " . $mode . "</p>");
		}
		
		//Implements a possibility to select future or present events
		//Thanks to Stig for this.
		if($parentObject->conf['mbl_newsevent.']['timeperiodSelMode'] == 1 || $parentObject->conf['mbl_newsevent.']['timeperiodSelMode'] == 'past') {
			$sqlinsert = " AND tx_mblnewsevent_from > " . time();
		} elseif($parentObject->conf['mbl_newsevent.']['timeperiodSelMode'] == 2 || $parentObject->conf['mbl_newsevent.']['timeperiodSelMode'] == 'future') {
			$sqlinsert = " AND tx_mblnewsevent_from < " . time();
		}
		
		
		$selectConf = array();
		$selectConf["selectFields"] = 'DISTINCT ' . $mode . '(FROM_UNIXTIME(tx_mblnewsevent_from)), ' .
		                              'YEAR(FROM_UNIXTIME(tx_mblnewsevent_from)) as year, *';
		$selectConf["where"] = "1=1 AND tx_mblnewsevent_isevent = 1 " . $sqlinsert . $parentObject->cObj->enableFields(
		  'tt_news', //The table for which to get the where clause
		  false //If set, then you want NOT to filter out hidden records. Otherwise hidden record are filtered based on the current preview settings.
		);
		$selectConf["pidInList"] =$parentObject->pid_list;
		$selectConf["orderBy"] = "year " . strtoupper($parentObject->conf['mbl_newsevent.']['dateYearSelOrder']) . ", mode " . strtoupper($parentObject->conf['mbl_newsevent.']['dateSelOrder']);
			
		$res = $this->cObj->exec_getQuery('tt_news',$selectConf);
		
		//Create list of items
		$items = array();
		
		while($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res)) {
			$markerArray = array();
			
			//If the number of articles should be counted
			if($parentObject->conf['mbl_newsevent.']['showEventPeriodCount'] != 0) {
				$selectConf = array();
				$selectConf["selectFields"] = 'COUNT(DISTINCT(uid)) as rowcount';
				$selectConf["where"] = "1=1 AND YEAR(FROM_UNIXTIME(tx_mblnewsevent_from)) = " . $row['year'] . 
				                       " AND $mode(FROM_UNIXTIME(tx_mblnewsevent_from)) = " . $row['mode'] . $parentObject->cObj->enableFields(
				  'tt_news', //The table for which to get the where clause
				  false //If set, then you want NOT to filter out hidden records. Otherwise hidden record are filtered based on the current preview settings.
				);
				$selectConf["pidInList"] =$parentObject->pid_list;
				
				$res2 = $this->cObj->exec_getQuery('tt_news',$selectConf);
				$row2 = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res);
				
				$markerArray["###EVENT_PERIOD_COUNT###"] = $row2['rowcount'];
			} else {
				$markerArray["###EVENT_PERIOD_COUNT###"] = "";
			}
			
			//Create the period name and link it
			if($mode == 'month') {
				$name = $parentObject->local_cObj->stdWrap(mktime(0, 0, 0, $row['mode'], 1, $row['year']), $parentObject->conf["mbl_newsevent."]["dateSelItem_stdWrap."]);
				$markerArray['###EVENT_PERIOD_NAME###'] = $this->pi_linkToPage(
				  $name, //The content string to wrap in tags
				  (($parentObject->conf["mbl_newsevent."]["eventPeriodPID"] != 0)?$parentObject->conf["mbl_newsevent."]["eventPeriodPID"]:$GLOBALS['TSFE']->id), //Page id
				  '', //Target value to use. Affects the &type-value of the URL, defaults to current.
				  array($this->prefixId.'[datemode]'=>$mode,
				        $this->prefixId.'[datesel]'=>date("m-Y", mktime(0, 0, 0, $row['mode'], 1, $row['year']))) //Additional URL parameters to set (key/value pairs)
				);
			} elseif($mode == 'quarter') {
				$name = $this->pi_getLL("quarter" . $row['mode']) . " " . $row['year'];
				$markerArray['###EVENT_PERIOD_NAME###'] = $this->pi_linkToPage(
				  $name, //The content string to wrap in tags
				  (($parentObject->conf["mbl_newsevent."]["eventPeriodPID"] != 0)?$parentObject->conf["mbl_newsevent."]["eventPeriodPID"]:$GLOBALS['TSFE']->id), //Page id
				  '', //Target value to use. Affects the &type-value of the URL, defaults to current.
				  array($this->prefixId.'[datemode]'=>$mode,
				        $this->prefixId.'[datesel]'=>$row['mode']."-".date("Y", mktime(0, 0, 0, $row['mode'], 1, $row['year']))) //Additional URL parameters to set (key/value pairs)
				);
			} else /*YEAR*/ {
				$markerArray['###EVENT_PERIOD_NAME###'] = $row['year'];
				$markerArray['###EVENT_PERIOD_VALUE###'] = mktime(0, 0, 0, 1, 1, $row['year']) . "-" . (mktime(0, 0, 0, 1, 1, $row['year']+1)-1);
				$name = $row['year'];
				$markerArray['###EVENT_PERIOD_NAME###'] = $this->pi_linkToPage(
				  $name, //The content string to wrap in tags
				  (($parentObject->conf["mbl_newsevent."]["eventPeriodPID"] != 0)?$parentObject->conf["mbl_newsevent."]["eventPeriodPID"]:$GLOBALS['TSFE']->id), //Page id
				  '', //Target value to use. Affects the &type-value of the URL, defaults to current.
				  array($this->prefixId.'[datemode]'=>$mode,
				        $this->prefixId.'[datesel]'=>$row['year']) //Additional URL parameters to set (key/value pairs)
				);
			}
			//Include the old value, but empty it, just to be sure it doesn't pop up :-D
			$markerArray['###EVENT_PERIOD_VALUE###'] = '';
			
			$items .= $parentObject->cObj->substituteMarkerArray($itemTemplate, $markerArray);
		}
		
		$template = $parentObject->cObj->substituteSubpart($template, '###ITEM###', $items)
		
		return array('###EVENT_SELECT_LIST###'=>$template, '###EVENT_SELECT_MENU###'=>'');
	}
	
	function oldmakeDateSel($parentObject) {
		//Get template
		$eventTemplateCode = $parentObject->cObj->fileResource(((empty($parentObject->conf["mbl_newsevent."]["templateFile"]))? $parentObject->conf["eventTemplateFile"] : $parentObject->conf["mbl_newsevent."]["templateFile"]));
		//Get template sections
		$dateSelMenuCode = $parentObject->cObj->getSubpart($eventTemplateCode,"###DATESEL_MENU###"); //For a menu
		$dateSelListCode = $parentObject->cObj->getSubpart($eventTemplateCode,"###DATESEL_LIST###"); //For a list
		//Get template section elements
		$dateSelMenuElement =  $parentObject->cObj->getSubpart($dateSelMenuCode,"###ITEM###"); //For menu
		$dateSelListElement =  $parentObject->cObj->getSubpart($dateSelListCode,"###ITEM###"); //For list
		
		//Select the right date mode either from conf or from hardcoded constant
		$switch = $parentObject->conf['mbl_newsevent.']['dateSelMode'];
		
		switch(strtolower($switch)) {
			//Show years
			case 'year':
				$mode = 'YEAR';
				break;
			//Show quarter
			case 'quarter':
				$mode = 'QUARTER';
				break;
			//Show month
			case 'month':
			default:
				$mode = 'MONTH';
				break;
		}
		
		//Implements a possibility to select future or present events
		//Thanks to Stig for this.
		if($parentObject->conf['mbl_newsevent.']['timeperiodSelMode'] == 1) {
			$sqlinsert = " AND tx_mblnewsevent_from > UNIX_TIMESTAMP()";
		} elseif($parentObject->conf['mbl_newsevent.']['timeperiodSelMode'] == 2) {
			$sqlinsert = " AND tx_mblnewsevent_from < UNIX_TIMESTAMP()";
		}
		
		//SQL with inserted function name
		$sql = "SELECT DISTINCT 
		            $mode(FROM_UNIXTIME(tx_mblnewsevent_from)) as mode,
		            YEAR(FROM_UNIXTIME(tx_mblnewsevent_from)) as year, pid, deleted
		        FROM
		            tt_news
		        WHERE
		            tx_mblnewsevent_isevent = 1 " . $sqlinsert . " AND pid IN (" . $parentObject->pid_list . ") AND deleted = 0
		        ORDER BY
		            year " . strtoupper($parentObject->conf['mbl_newsevent.']['dateYearSelOrder']) . ", mode " . strtoupper($parentObject->conf['mbl_newsevent.']['dateSelOrder']);
		
		$res = mysql(TYPO3_db, $sql);
				
		//Create array items
		$menu = '<option value="">' . $this->pi_getLL("select_date") . '</option>'; //Insert default option aswell for JavaScript reasons (onchange doesn't work if only one item
		$list = "";
		while($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res)) {
			//Create marker array
			$markerArray = array();
			
			//If the number of articles should be counted
			if($parentObject->conf['mbl_newsevent.']['showEventPeriodCount'] != 0) {
				//Count the number of articles
				$sql = "SELECT COUNT(DISTINCT(uid)) as rowcount FROM tt_news WHERE YEAR(FROM_UNIXTIME(tx_mblnewsevent_from)) = " . $row['year'] . " AND $mode(FROM_UNIXTIME(tx_mblnewsevent_from)) = " . $row['mode'];
				
				$countRes = mysql(TYPO3_db, $sql);
				$countRow = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($countRes);
				$count = $countRow['rowcount'];

				$markerArray["###EVENT_PERIOD_COUNT###"] = $count;
			} else {
				$markerArray["###EVENT_PERIOD_COUNT###"] = "";
			}
			
			//Fill marker array
			//Needs a bit of special treatment for handeling quarters
			if($mode == 'MONTH') {
				$markerArray['###EVENT_PERIOD_NAME###'] = $parentObject->local_cObj->stdWrap(mktime(0, 0, 0, $row['mode'], 1, $row['year']), $parentObject->conf["mbl_newsevent."]["dateSelItem_stdWrap."]);
				$markerArray['###EVENT_PERIOD_VALUE###'] = mktime(0, 0, 0, $row['mode'], 1, $row['year']) . "-" . (mktime(0, 0, 0, $row['mode']+1, 1, $row['year'])-1);
			} elseif($mode == 'QUARTER') {
				$markerArray['###EVENT_PERIOD_NAME###'] =  $this->pi_getLL("quarter" . $row['mode']) . " " . $row['year'];
				$markerArray['###EVENT_PERIOD_VALUE###'] = mktime(0, 0, 0, ($row['mode'])*3, 1, $row['year']) . "-" . mktime(0, 0, 0, ($row['mode']+1)*3, 1, $row['year']);
				
			} else /*YEAR*/ {
				$markerArray['###EVENT_PERIOD_NAME###'] = $row['year'];
				$markerArray['###EVENT_PERIOD_VALUE###'] = mktime(0, 0, 0, 1, 1, $row['year']) . "-" . (mktime(0, 0, 0, 1, 1, $row['year']+1)-1);
			}
			
			$markerArray['###EVENT_PERIOD_LINK###'] = $parentObject->pi_getPageLink((($parentObject->conf["mbl_newsevent."]["eventPeriodPID"] != 0)?$parentObject->conf["mbl_newsevent."]["eventPeriodPID"]:$GLOBALS['TSFE']->id)) . "&";
			$markerArray['###EVENT_PERIOD_TARGET###'] = $parentObject->conf["mbl_newsevent."]["eventPeriodTarget"];
			$markerArray['###EVENT_PERIOD_FIELD###'] = $parentObject->prefixId.'[datesel]';//"tx_mblnewsevent_datesel";
			$markerArray['###EVENT_PERIOD_HIDDEN###'] = '<input type="hidden" name="' . parentOpbject->prefixId.'[dateperiod]' . ' value="' . strtolower($mode) . '" />';
			
			//Insert for menu
			$menu .= $parentObject->cObj->substituteMarkerArray($dateSelMenuElement, $markerArray);
			//Insert for list
			$list .= $parentObject->cObj->substituteMarkerArray($dateSelListElement, $markerArray);
		}
		
		//Make general markerArray
		$generalMarkerArray = array();
		$generalMarkerArray['###EVENT_PERIOD_LINK###'] = $markerArray['###EVENT_PERIOD_LINK###'];
		$generalMarkerArray['###EVENT_PERIOD_TARGET###'] = $markerArray['###EVENT_PERIOD_TARGET###'];
		$generalMarkerArray['###EVENT_PERIOD_FIELD###'] = $markerArray['###EVENT_PERIOD_FIELD###'];
		$generalMarkerArray['###EVENT_PERIOD_HIDDEN###'] = $markerArray['###EVENT_PERIOD_HIDDEN###'];

		//Replace general
		//Insert for menu
		$dateSelMenuCode = $parentObject->cObj->substituteMarkerArray($dateSelMenuCode, $generalMarkerArray);
		//Insert for list
		$dateSelListCode = $parentObject->cObj->substituteMarkerArray($dateSelListCode, $generalMarkerArray);
		
		//Return
		return array('###EVENT_SELECT_LIST###'=>$parentObject->cObj->substituteSubpart($dateSelListCode, '###ITEM###', $list)
		             );
	}
	
	function extraCodesProcessor($parentObject) {
		//Initiate language
		$this->pi_loadLL();
		//print_r($parentObject->theCode);
		$content = "";
		switch($parentObject->theCode) {
			case "EVENT_FUTURE":
			case "EVENT_PAST":
			case "LATEST_EVENT_PAST":
			case "LATEST_EVENT_FUTURE":
				$content = $this->listNewsEvent($parentObject);
				break;
			case "EVENT_SELECTOR":
				$menu = $this->makeDateSel($parentObject);
				if(trim(strtolower($parentObject->conf["mbl_newsevent."]["eventSelector"])) == "list") {
					$content = $menu['###EVENT_SELECT_LIST###'];
				} else {
					$content = $menu['###EVENT_SELECT_MENU###'];
				}
				break;
			case "ICS":
				$content = $this->icsOutput($parentObject);
				break;
		}
		
		return $content;
	}

	function listNewsEvent($parentObject) {
		//Decide wether to show future or past events
		$where=" AND tt_news.tx_mblnewsevent_isevent = 1";
		$templateName = null;
		
		$orderByString = null;
		$order = "DESC";
		switch($parentObject->theCode) {
			//FUTURE
			case "LATEST_EVENT_FUTURE":
				if(intval($parentObject->conf["latestLimit"]))	$parentObject->config["limit"] = intval($parentObject->conf["latestLimit"]);
				if (intval($parentObject->conf["latestLimit"]))	$parentObject->config["limit"] = intval($parentObject->conf["latestLimit"]);
				$parentObject->arcExclusive = -1; // Only latest, non archive news
				$parentObject->config['limit'] = $parentObject->config['latestLimit'];
				$templateName = "TEMPLATE_LATEST";
				$prefix_display="displayLatest";
				//Makes the list show new events and not just 
				if($parentObject->conf['mbl_newsevent.']['latestShowsNewEvents'] == 1) {
					$orderByString = "tt_news.crdate DESC";
				}
				$order = "ASC";
			case "EVENT_FUTURE":
				$where .= " AND tt_news.tx_mblnewsevent_from >= " . time();
				if(!$templateName) { $templateName = "TEMPLATE_LIST"; }
				if($parentObject->theCode == "EVENT_FUTURE") { $prefix_display="displayList"; } 
				break;
			//PAST EVENTS
			case "LATEST_EVENT_PAST":
				if(intval($parentObject->conf["latestLimit"]))	$parentObject->config["limit"] = intval($parentObject->conf["latestLimit"]);
				if (intval($parentObject->conf["latestLimit"]))	$parentObject->config["limit"] = intval($parentObject->conf["latestLimit"]);
				$templateName = "TEMPLATE_LATEST";
				$prefix_display="displayLatest";
				//Makes the list show new events and not just 
				if($parentObject->conf['mbl_newsevent.']['latestShowsNewEvents'] == 1) {
					$orderByString = "tt_news.crdate DESC";
				}
				$order = "DESC";
			case "EVENT_PAST":
				$where .= " AND tt_news.tx_mblnewsevent_from <= " . time();
				if(!$templateName) { $templateName = "TEMPLATE_LIST"; }
				if($parentObject->theCode == "EVENT_PAST") { $prefix_display="displayList"; } 
				break;
		}		
		
		//limit to event period
		if(!empty($parentObject->piVars['datesel'])) {
			$fromto = explode("-", $parentObject->piVars['datesel']);
			$where .= " AND tt_news.tx_mblnewsevent_from >= " . $fromto[0] . " AND tt_news.tx_mblnewsevent_from <= " . $fromto[1];
		}
		
		$theCode = $parentObject->theCode;
		$content = "";
		
		$noPeriod = 0;

		if (!$parentObject->config['emptyArchListAtStart']) {
			// if this is true, we're listing from the archive for the first time (no pS set), to prevent an empty list page we set the pS value to the archive start
            if (($parentObject->arcExclusive > 0 && !$parentObject->piVars['pS'] && $theCode != 'SEARCH')) {
				// set pS to time minus archive startdate
				$parentObject->piVars['pS'] = ($GLOBALS['SIM_EXEC_TIME']-($parentObject->config['datetimeDaysToArchive']*86400));
			}
		}

		if ($parentObject->piVars['pS'] && !$parentObject->piVars['pL']) {
			$noPeriod = 1; // override the period lenght checking in getSelectConf
		}

		// Allowed to show the listing? periodStart must be set, when listing from the archive.
		if (!($parentObject->arcExclusive > -1 && !$parentObject->piVars['pS'] && $theCode != 'SEARCH')) {

			if ($parentObject->config['displayCurrentRecord'] && $parentObject->tt_news_uid) {
				$parentObject->pid_list = $parentObject->cObj->data['pid'];
				$where = 'AND tt_news.uid='.$parentObject->tt_news_uid;
			}

			// build parameter Array for List query
			$selectConf = $parentObject->getSelectConf($where, $noPeriod);

			// performing query to count all news (we need to know it for browsing):
			$selectConf['selectFields'] = 'count(distinct(uid))'; //count(*)
			$res = $parentObject->cObj->exec_getQuery('tt_news', $selectConf);
			$row = $GLOBALS['TYPO3_DB']->sql_fetch_row($res);
			$newsCount = $row[0];

			//Only do something if the queryresult is not empty
			if ($newsCount > 0) {
				// Init Templateparts: $t['total'] is complete template subpart (TEMPLATE_LATEST f.e.)
				// $t['item'] is an array with the alternative subparts (NEWS, NEWS_1, NEWS_2 ...)
				$t = array();
				$t['total'] = $parentObject->cObj->getSubpart($parentObject->templateCode, $parentObject->spMarker('###'.$templateName.'###'));
				$t['item'] = $parentObject->getLayouts($t['total'], $parentObject->alternatingLayouts, 'NEWS');

				// build query for display:
				$selectConf['groupBy'] = 'uid';
				if(empty($orderByString)) {
					$selectConf['orderBy'] = 'tt_news.tx_mblnewsevent_from ' . $order;
				} else {
					$selectConf['orderBy'] = $orderByString;
				}
				$selectConf['selectFields'] = '*';
				#$selectConf['max'] = intval($parentObject->config['limit']+1);

				// exclude the LATEST template from changing its content with the pagebrowser. This can be overridden by setting the conf var latestWithPagebrowser
				if ($theCode != 'LATEST' && !$parentObject->conf['latestWithPagebrowser']) {
					//If mbl_newsevent settings should be used
					if($parentObject->conf['mbl_newsevent.']['limit'] != 0) {
						$selectConf['begin'] = $parentObject->piVars['pointer']*$parentObject->conf['mbl_newsevent.']['limit'];
					} else {
						$selectConf['begin'] = $parentObject->piVars['pointer']*$parentObject->config['limit'];
					}
				}
				// exclude news-records shown in LATEST from the LIST template
				if ($theCode == 'LIST' && $parentObject->conf['excludeLatestFromList'] && !$parentObject->piVars['pointer'] && !$parentObject->piVars['cat']) {
					if  ($parentObject->conf['latestLimit']) {
							$selectConf['begin'] += (($parentObject->conf['mbl_newsevent.']['latestLimit'] != 0)? $parentObject->conf['mbl_newsevent.']['latestLimit'] : $parentObject->conf['latestLimit']);
							$newsCount -= $parentObject->conf['latestLimit'];
					} else {
						$selectConf['begin'] += $newsCount;
						// this will clean the display of LIST view when 'latestLimit' is unset because all the news have been shown in LATEST already
					}
				}

				// Reset:
				$subpartArray = array();
				$wrappedSubpartArray = array();
				$markerArray = array();

				// get the list of news items and fill them in the CONTENT subpart
				$subpartArray['###CONTENT###'] = $parentObject->getListContent($t['item'], $selectConf, $prefix_display);

				$markerArray['###GOTOARCHIVE###'] = $parentObject->pi_getLL('goToArchive');
				$markerArray['###LATEST_HEADER###'] = $parentObject->pi_getLL('latestHeader');
				$wrappedSubpartArray['###LINK_ARCHIVE###'] = $parentObject->local_cObj->typolinkWrap($parentObject->conf['archiveTypoLink.']);

				// unset previous and next link
				$markerArray['###LINK_PREV###'] = '';
				$markerArray['###LINK_NEXT###'] = '';
				// render a pagebrowser if needed
				
				if ($newsCount>$parentObject->config['limit']) {
					// configure pagebrowser
    				$parentObject->internal['res_count'] = $newsCount;
					$parentObject->internal['results_at_a_time'] = $parentObject->config['limit'];
					$parentObject->internal['maxPages'] = $parentObject->config['pageBrowser.']['maxPages'];
					if (!$parentObject->config['pageBrowser.']['showPBrowserText']) {
					    $parentObject->LOCAL_LANG[$parentObject->LLkey]['pi_list_browseresults_page'] = ''; 
					}
					
					$markerArray['###BROWSE_LINKS###'] = $parentObject->pi_list_browseresults($parentObject->config['pageBrowser.']['showResultCount'],$parentObject->config['pageBrowser.']['tableParams']);
				} else {
					$markerArray['###BROWSE_LINKS###'] = '';
				}

				$content .= $parentObject->cObj->substituteMarkerArrayCached($t['total'], $markerArray, $subpartArray, $wrappedSubpartArray);
			} elseif (ereg('1=0', $where)) {
				// first view of the search page with the parameter 'emptySearchAtStart' set
				$markerArray['###SEARCH_EMPTY_MSG###'] = $parentObject->local_cObj->stdWrap($parentObject->pi_getLL('searchEmptyMsg'), $parentObject->conf['searchEmptyMsg_stdWrap.']);
				$searchEmptyMsg = $parentObject->cObj->getSubpart($parentObject->templateCode, $parentObject->spMarker('###TEMPLATE_SEARCH_EMPTY###'));

				$content .= $parentObject->cObj->substituteMarkerArrayCached($searchEmptyMsg, $markerArray);
			} elseif ($parentObject->piVars['swords']) {
				// no results
				$markerArray['###SEARCH_EMPTY_MSG###'] = $parentObject->local_cObj->stdWrap($parentObject->pi_getLL('noResultsMsg'), $parentObject->conf['searchEmptyMsg_stdWrap.']);
				$searchEmptyMsg = $parentObject->cObj->getSubpart($parentObject->templateCode, $parentObject->spMarker('###TEMPLATE_SEARCH_EMPTY###'));
				$content .= $parentObject->cObj->substituteMarkerArrayCached($searchEmptyMsg, $markerArray);
			} else {
				$content .= $parentObject->local_cObj->stdWrap($parentObject->pi_getLL('noNewsToListMsg'), $parentObject->conf['noNewsToListMsg_stdWrap.']);
			}
		}
		
		
		return $content;
	}
}

?>